/*
 * Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Additionally, if used for any scientific purpose, the following reference
 * must be cited:
 * 
 * Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
 * 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
 * Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
 * pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef PSO_HPP_
#define PSO_HPP_

#include <sys/time.h>
#include <float.h>
#include <stdlib.h>
#include "ISearch.hpp"

#define PI (3.141592653589793238462643383279)

#define COPY_ARR(orig, dest, sz) for(int _i_=0; _i_ < sz; dest[_i_] = orig[_i_], _i_++);

#define RAND_DOUBLE(seed, a, b) ((a==b)?a:(a+(((double)rand_r(&seed))/RAND_MAX)*(b-a)))

typedef double (*callback_t)(double *buf, int n);

/**
 * @brief Particle Swarm Optimization.
 * 
 * @date 04/Mar/2017
 * @author Peter Frank Perroni (pfperroni@gmail.com)
 */
class PSO : public ISearch{
	unsigned int seed;
	bool decreasingW;
	int p, n, nEvals, gb;
	double **x, *fit, s1, s2, w, c1, c2, *Gb, Gb_Fit, **pb, *pb_Fit, **v;

	callback_t fitnessFunction;

public:
	PSO(callback_t fitnessFunction, double s1, double s2, int p, int n, double w, double c1, double c2) ;
	~PSO();
	void startup();
	void next(int M);
	int getBestPos(double *_x);
	int getNEvals();
	double getFitness();
	unsigned int getRandomSeed();
};

#endif /* PSO_HPP_ */
