# Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# Additionally, if used for any scientific purpose, the following reference
# must be cited:
# 
# Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
# 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
# Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
# pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# pso_example.cpp
#
# Created on: 02/Jul/2017
# Author: Peter Frank Perroni (pfperroni@gmail.com)
#
# This file contains an example of how to call the CSMOn C++ API.

from CSMOn import *

# Prepare the Input parameters.
bestResult=[Param(b"nEvals", i=0), Param(b"fit", d=0)]
# PSO Parameters.
s1=Param(b"s1", d=-5.12)
s2=Param(b"s2", d=5.12)
w=Param(b"w", d=-0.5)
c1=Param(b"c1", d=0.2)
c2=Param(b"c2", d=0.35)
n=Param(b"n", i=100)
p=Param(b"p", i=30)
# CSMOn parameters.
M=Param(b"M", i=50000) # Maximum number of evaluations allowed.
R=Param(b"R", d=-0.08) # Relaxation sensibility.

# Call CSMOn running a PSO.
res=search(b"pso", [s1, s2, w, c1, c2, n, p, M, R], bestResult)

#Print the results.
print("\n", "Optimized result: ", res)
print('\n', "nEvals:", bestResult[0].i, ", Fitness:", bestResult[1].d)
